
import os

import cdsapi
import numpy as np
import pandas
import xarray


def calculate_wind_direction(u, v):
    return (270.0 - np.rad2deg(np.arctan2(v,u))) % 360


def get_wind(time, lon, lat, data_path, components='uv'):
    
    p, u, v = get_era5_wind_profile(time, lon, lat, data_path=data_path)
    
    # TODO: use weights
    u = u.mean('level')
    v = v.mean('level')
        
    if components == 'uv':
        return u, v
    
    else:
        wind_speed = np.sqrt(u**2 + v**2)
        wind_direction = calculate_wind_direction(u, v)
        return wind_speed, wind_direction
    

def download_era5_winds(time, data_path='.', overwrite=False):
    
    era5_filename = os.path.join(data_path, time.strftime('ERA5-wind-%Y%m%d_%H%M.nc'))
    
    if os.path.exists(era5_filename) and not overwrite:
        return
    
    
    cds = cdsapi.Client()

    r = cds.retrieve(
        'reanalysis-era5-pressure-levels',
        {
            'product_type': 'reanalysis',
            'format': 'netcdf',
            'variable': [
                'u_component_of_wind', 'v_component_of_wind',
                'geopotential',
            ],
            'pressure_level': [
                '800', '825', '850',
                '875', '900', '925',
                '950', '975', '1000',
            ],
            "date": time.strftime('%Y-%m-%d'),
            "time": time.strftime('%H:00'),
            "area": [90, -180, -90, 180], # north, east, south, west
        }, era5_filename
    )
    

def get_era5_wind_profile(time, lon, lat, data_path='.'):

    era5_filename = os.path.join(data_path, time.strftime('ERA5-wind-%Y%m%d_%H%M.nc'))
    download_era5_winds(time, data_path, overwrite=False)

    with xarray.open_dataset(era5_filename) as era5:
        era5 = era5.load().sel(time=time, longitude=lon, latitude=lat, method='nearest')

    pressure = era5['level']
    u_profile = era5['u']
    v_profile = era5['v']  
    
    return pressure, u_profile, v_profile



def calculate_pressure_levels(surface_pressure, nlevels=37, which='middle'):
    
    filename= os.path.join(os.path.dirname(__file__), 'data',
                           'ERA5 L137 model level definitions.csv')
    
    level_definitions = pandas.read_csv(filename, index_col=0)

    a = np.array(level_definitions['a [Pa]'])
    b = np.array(level_definitions['b'])
    
    # pressure at boundary of model layers
    ph = a + b * float(surface_pressure)
   
    # TODO: how to interpolate?
    #ph = np.interp(np.linspace(0,138,nlevels+1), np.arange(138), ph)

    # pressure at middle of model layers
    pf = 0.5 * (ph[1:] + ph[:-1])
    
    if which == 'middle':
        return pf
    elif which == 'boundary':
        return ph
    else:
        raise ValueError


def calculate_level_heights(temperature, pressure, which='middle'):
    """
    temperature at middle of layers
    pressure at boundary of layers
    """
    g = 9.80665
    R = 287.06
  
    
    p1 = pressure[1:]
    p2 = pressure[:-1]

    dh = R * temperature / g * np.log(p1 / p2)
    
    hh = np.concatenate([[0.0], dh[::-1].cumsum()])
    hh = hh[::-1]
    
    hf = 0.5 * (hh[1:] + hh[:-1])
    
    if which == 'middle':
        return hf
    elif which == 'boundary':
        return hh
    else:
        raise ValueError
    
    
